//
// spring.cpp  written by Kenji Shimada
// Problem Set 7 Sample Code 
//

#include  "spring.h"


void vrml_path (ofstream &outFile, int n, double p[][3]) {
// Draws the function defined in ftheta on the canvas
//  outFile: pointer to the output vrml file
//  n:       the number of iteration
//  p:       2x2 matrix that stores the path

	int			i;

	outFile << "# Draw 2D path\n";
	outFile << "#-----------------------------------\n";
	outFile << "Transform {\n";
	outFile << "\ttranslation 0 0 0";
	outFile << "\tchildren Shape {\n";
	outFile << "\t\tappearance Appearance {\n";
	outFile << "\t\t\tmaterial Material {\n";
	outFile << "\t\t\t\tdiffuseColor 0 0 1\n\t\t\t}\n\t\t}\n";
	outFile << "\t\tgeometry Extrusion {\n";
	outFile << "\t\t\tcrossSection [1 0, 0 -1, -1 0, 0 1, 1 0]\n";
	outFile << "\t\t\tscale ["<<DRAW_WIDTH<<" "<<DRAW_WIDTH<<"]\n";
	outFile << "\t\t\tspine [";
	for (i=0; i<n; i++)
		outFile << p[i][1]*100 << " " << -7.0 << " " << p[i][0]*100 << ", ";
	outFile << "]\n";
	outFile << "\t\t}\n\t}\n}\n";

	outFile << "# Draw 3D path\n";
	outFile << "#-----------------------------------\n";
	outFile << "Transform {\n";
	outFile << "\ttranslation 0 0 0";
	outFile << "\tchildren Shape {\n";
	outFile << "\t\tappearance Appearance {\n";
	outFile << "\t\t\tmaterial Material {\n";
	outFile << "\t\t\t\tdiffuseColor 1 0 0\n\t\t\t}\n\t\t}\n";
	outFile << "\t\tgeometry Extrusion {\n";
	outFile << "\t\t\tcrossSection [1 0, 0 -1, -1 0, 0 1, 1 0]\n";
	outFile << "\t\t\tscale ["<<DRAW_WIDTH<<" "<<DRAW_WIDTH<<"]\n";
	outFile << "\t\t\tspine [";
	for (i=0; i<n; i++)
		outFile << p[i][1]*100 << " " << p[i][2]*3.0 << " " << p[i][0]*100 << ", ";
	outFile << "]\n";
	outFile << "\t\t}\n\t}\n}\n";
}


void vrml_surface (ofstream &outFile, char* my_name) {
// Draw in the output vrml file, canvas, background, heading, and surface
//  outFile: pointer to the output vrml file
//  my_name: my name to be shown in the heading
	
	int			i, j;

	// Create Header
	outFile << "#VRML V2.0 utf8\n";
	outFile << "#------------------------------------------------\n";
	outFile << "# 24-311: Numerical Methods  Fall'98\n";
	outFile << "# "<<MY_NAME<<"\n";
	outFile << "# 2D Unconstrained Optimization\n";
	outFile << "#------------------------------------------------\n\n";

	// Set up background
	outFile << "# Specify the background color\n";
	outFile << "#-----------------------------------\n";
	outFile << "Background {\n\tskyColor 1 1 1\n}\n\n"; // White background

	// Create the square for the canvas
	outFile << "# Draw the first canvas\n";
	outFile << "#-----------------------------------\n";
	outFile << "Transform {\n";
	outFile << "\trotation 1 0 0 1.5707\n";
	outFile << "\ttranslation 10 0 10\n";
	outFile << "\tchildren Shape {\n";
	outFile << "\t\tappearance Appearance {\n";
	outFile << "\t\t\tmaterial Material {\n";
	outFile << "\t\t\t\tdiffuseColor 0.9 0.2 0.2\n";
	outFile << "\t\t\t\ttransparency 0.5\n\t\t\t}\n\t\t}\n";
	outFile << "\t\tgeometry Box {\n";
	outFile << "\t\t\tsize 20 20 0.1\n\t\t}\n\t}\n}\n\n";

	// Create the square for the canvas
	outFile << "# Draw the second canvas\n";
	outFile << "#-----------------------------------\n";
	outFile << "Transform {\n";
	outFile << "\trotation 1 0 0 1.5707\n";
	outFile << "\ttranslation 10 -7.2 10\n";
	outFile << "\tchildren Shape {\n";
	outFile << "\t\tappearance Appearance {\n";
	outFile << "\t\t\tmaterial Material {\n";
	outFile << "\t\t\t\tdiffuseColor 0.2 0.2 0.9\n";
	outFile << "\t\t\t\ttransparency 0.5\n\t\t\t}\n\t\t}\n";
	outFile << "\t\tgeometry Box {\n";
	outFile << "\t\t\tsize 20 20 0.1\n\t\t}\n\t}\n}\n\n";

	// Put text heading on the VRML screen
	outFile << "# Draw texts\n";
	outFile << "#-----------------------------------\n";
	outFile << "Transform {\n";
	outFile << "\ttranslation 10 18 2\n";
	outFile << "\tchildren Shape {\n";
	outFile << "\t\tappearance Appearance {\n";
	outFile << "\t\t\tmaterial Material {\n";
	outFile << "\t\t\t\tdiffuseColor 1 0 0\n";
	outFile << "\t\t\t}\n\t\t}\n";
	outFile << "\t\tgeometry Text {\n";
	outFile << "\t\t\tstring [\"24-311: Numerical Methods--PS7\",\n";
	outFile << "\t\t\t\"2D Unconstrained Optimization\",\n";
	outFile << "\t\t\t\""<<MY_NAME<<"\"]\n";
	outFile << "\t\t\tfontStyle FontStyle {\n";
	outFile << "\t\t\t\tfamily \"SANS\"\n";
	outFile << "\t\t\t\tjustify \"MIDDLE\"\n";
	outFile << "\t\t\t\tstyle \"BOLD\"\n";
	outFile << "\t\t\t\tsize 1.5\n\t\t\t}\n\t\t}\n\t}\n}\n\n";

	// Add the x-axis
	outFile << "# Define the x axis\n";
	outFile << "#-----------------------------------\n";
	outFile << "Transform {\n";
	outFile << "\ttranslation 0 0 0\n";
	outFile << "\tchildren DEF AXIS Shape {\n";
	outFile << "\t\tappearance Appearance {\n";
	outFile << "\t\t\tmaterial Material {\n";
	outFile << "\t\t\t\tdiffuseColor 0.0 0.0 0.0\n\t\t\t}\n\t\t}\n";
	outFile << "\t\tgeometry Extrusion {\n";
	outFile << "\t\t\tcrossSection [1 0, 0 -1, -1 0, 0 1, 1 0]\n";
	outFile << "\t\t\tscale [0.1 0.1, 0.1 0.1, 0.4 0.4, 0 0]\n";
	outFile << "\t\t\tspine [0 0 0, 21 0 0, 21 0 0, 22 0 0]\n\t\t}\n\t}\n}\n\n";

	// Add the y-axis
	outFile << "# Draw the y axis\n";
	outFile << "#-----------------------------------\n";
	outFile << "Transform {\n";
	outFile << "\trotation 0 0 1 1.5707\n";
	outFile << "\tchildren USE AXIS\n}\n\n";

	// Add the z-axis
	outFile << "# Draw the z axis\n";
	outFile << "#-----------------------------------\n";
	outFile << "Transform {\n";
	outFile << "\trotation 0 1 0 -1.5707\n";
	outFile << "\tchildren USE AXIS\n}\n\n";

	// Draw the surface
	outFile << "# Draw the surface\n";
	outFile << "#-----------------------------------\n";
	outFile << "Transform {\n";
	outFile << "\ttranslation 0 0 0\n";
	outFile << "\tchildren Shape {\n";
	outFile << "\t\tappearance Appearance {\n";
	outFile << "\t\t\tmaterial Material {\n";
	outFile << "\t\t\t\tdiffuseColor 1.0 1.0 0.0\n";
	outFile << "\t\t\t\ttransparency 0.25\n";
	outFile << "\t\t\t}\n\t\t}\n";
	outFile << "\t\tgeometry ElevationGrid{\n";
	outFile << "\t\t\tsolid FALSE\n";
	outFile << "\t\t\txDimension " << SURF_PREC <<"\n";
	outFile << "\t\t\tzDimension " << SURF_PREC <<"\n";
	outFile << "\t\t\txSpacing " << X_SPACING <<"\n";
	outFile << "\t\t\tzSpacing " << Y_SPACING <<"\n";
	outFile << "\t\t\theight [";
	for (i=0; i<SURF_PREC; i++)
		for (j=0; j<SURF_PREC; j++)
			outFile << 3.0*energy(X_SPACING*i/100.0, Y_SPACING*j/100.0) << ", ";
	outFile << "]\n";
	outFile << "\n\t\t}\n\t}\n}\n\n";

}
